@@ -8,12 +8,12 @@
 
 SEXP setattrib(SEXP x, SEXP name, SEXP value)
 {
-  if (!isString(name) || LENGTH(name)!=1) error("Attribute name must be a character vector of length 1");
+  if (!isString(name) || LENGTH(name)!=1) error(_("Attribute name must be a character vector of length 1"));
   if (!isNewList(x) &&
       strcmp(CHAR(STRING_ELT(name,0)),"class")==0 &&
       isString(value) && LENGTH(value)>0 &&
       (strcmp(CHAR(STRING_ELT(value, 0)),"data.table")==0 || strcmp(CHAR(STRING_ELT(value,0)),"data.frame")==0) ) {
-    error("Internal structure doesn't seem to be a list. Can't set class to be 'data.table' or 'data.frame'. Use 'as.data.table()' or 'as.data.frame()' methods instead.");
+    error(_("Internal structure doesn't seem to be a list. Can't set class to be 'data.table' or 'data.frame'. Use 'as.data.table()' or 'as.data.frame()' methods instead."));
   }
   if (isLogical(x) && LENGTH(x)==1 &&
       (x==ScalarLogical(TRUE) || x==ScalarLogical(FALSE) || x==ScalarLogical(NA_LOGICAL))) {  // R's internal globals, #1281
@@ -22,11 +22,18 @@ SEXP setattrib(SEXP x, SEXP name, SEXP value)
     UNPROTECT(1);
     return(x);
   }
-  setAttrib(x, name,
-    MAYBE_REFERENCED(value) ? duplicate(value) : value);
+  if (isNull(value) && isPairList(x) && strcmp(CHAR(STRING_ELT(name,0)),"names")==0) {
+    // backport fix in R 3.2.0 to support R 3.1.0; #4048 #3802
+    // apply this backport always (i.e. in R >=3.2.0 too) to avoid a switch on version number or feature test (to avoid more code, tests and nocov)
+    for (SEXP t=x; t!=R_NilValue; t=CDR(t)) {
+      SET_TAG(t, R_NilValue);
+    }
+  } else {
+    setAttrib(x, name, MAYBE_REFERENCED(value) ? duplicate(value) : value);
     // duplicate is temp fix to restore R behaviour prior to R-devel change on 10 Jan 2014 (r64724).
     // TO DO: revisit. Enough to reproduce is: DT=data.table(a=1:3); DT[2]; DT[,b:=2]
     // ... Error: selfrefnames is ok but tl names [1] != tl [100]
+  }
   return(R_NilValue);
 }
 
@@ -56,10 +63,10 @@ SEXP setlistelt(SEXP l, SEXP i, SEXP value)
 {
   R_len_t i2;
   // Internal use only. So that := can update elements of a list of data.table, #2204. Just needed to overallocate/grow the VECSXP.
-  if (!isNewList(l)) error("First argument to setlistelt must be a list()");
-  if (!isInteger(i) || LENGTH(i)!=1) error("Second argument to setlistelt must a length 1 integer vector");
+  if (!isNewList(l)) error(_("First argument to setlistelt must be a list()"));
+  if (!isInteger(i) || LENGTH(i)!=1) error(_("Second argument to setlistelt must a length 1 integer vector"));
   i2 = INTEGER(i)[0];
-  if (LENGTH(l) < i2 || i2<1) error("i (%d) is outside the range of items [1,%d]",i2,LENGTH(l));
+  if (LENGTH(l) < i2 || i2<1) error(_("i (%d) is outside the range of items [1,%d]"),i2,LENGTH(l));
   SET_VECTOR_ELT(l, i2-1, value);
   return(R_NilValue);
 }
@@ -81,7 +88,7 @@ SEXP expandAltRep(SEXP x)
   // At R level (for example [.data.table) we use and benefit from altrep vectors very much. It's just as columns that we expand them.
   // See extensive discussion in issue #2866
 
-  if (TYPEOF(x) != VECSXP) error("x isn't a VECSXP");
+  if (TYPEOF(x) != VECSXP) error(_("x isn't a VECSXP"));
   for (int i=0; i<LENGTH(x); i++) {
     SEXP col = VECTOR_ELT(x,i);
     if (ALTREP(col)) {
@@ -96,7 +103,7 @@ SEXP dim(SEXP x)
   // fast implementation of dim.data.table
 
   if (TYPEOF(x) != VECSXP) {
-    error("dim.data.table expects a data.table as input (which is a list), but seems to be of type %s",
+    error(_("dim.data.table expects a data.table as input (which is a list), but seems to be of type %s"),
         type2char(TYPEOF(x)));
   }
 
